# from __future__ import print_function  # for print() in Python 2
from io import open
import sys
import re
import logging

from pysqlformatter.src.formatter import Formatter
from sparksqlformatter import Style as sparksqlStyle
from sparksqlformatter import api as sparksqlAPI

logger = logging.getLogger(__name__)
log_formatter = '[%(asctime)s] %(levelname)s [%(filePath)s:%(lineno)s:%(funcName)s] %(message)s'
logging.basicConfig(stream=sys.stdout, level=logging.INFO, format=log_formatter)


def format_file(filePath, pythonStyle='pep8', sparksqlStyle=sparksqlStyle(), queryNames=['query'], inPlace=False):
    '''
    Format file with given settings for python style and sparksql configurations.

    Parameters
    filePath: string
        Path to the file to format.
    pythonStyle: string
        A style name or path to a style config file; interface to https://github.com/google/yapf.
    sparksqlStyle: string, dict, or sparksqlformatter.src.style.Style() object
        Configurations for the query language; interface to https://github.com/largecats/sparksql-formatter.
    queryNames: list

    inPlace: bool
        If True, will format the file in place.
        Else, will write the formatted file to stdout.

    Return: None
    '''
    if type(sparksqlStyle) == type(sparksqlStyle):
        formatter = Formatter(pythonStyle=pythonStyle, sparksqlStyle=sparksqlStyle)
    else:
        if type(sparksqlStyle) == str:
            if sparksqlStyle.startswith('{'):
                sparksqlStyle = eval(sparksqlStyle)
                formatter = Formatter(pythonStyle=pythonStyle,
                                      sparksqlStyle=sparksqlAPI._create_style_from_dict(sparksqlStyle),
                                      queryNames=queryNames)
            else:
                formatter = Formatter(pythonStyle=pythonStyle,
                                      sparksqlStyle=sparksqlAPI._create_style_from_file(sparksqlStyle))
        elif type(sparksqlStyle) == dict:
            formatter = Formatter(pythonStyle=pythonStyle,
                                  sparksqlStyle=sparksqlAPI._create_style_from_dict(sparksqlStyle),
                                  queryNames=queryNames)
        else:
            raise Exception('Unsupported config type')
    _format_file(filePath, formatter, inPlace)


def format_script(script, pythonStyle='pep8', sparksqlStyle=sparksqlStyle(), queryNames=['query']):
    '''
    Format script using given settings for python style and sparksql configurations.

    Parameters
    script: string
        The script to be formatted.
    pythonStyle: string
        A style name or path to a style config file; interface to https://github.com/google/yapf.
    sparksqlStyle: string, dict, or sparksqlformatter.src.style.Style() object
        Configurations for the query language; interface to https://github.com/largecats/sparksql-formatter.
    
    Return: string
        The formatted script.
    '''
    if type(sparksqlStyle) == type(sparksqlStyle):
        formatter = Formatter(pythonStyle=pythonStyle, sparksqlStyle=sparksqlStyle, queryNames=queryNames)
    else:
        if type(sparksqlStyle) == str:
            if sparksqlStyle.startswith('{'):
                sparksqlStyle = eval(sparksqlStyle)
                formatter = Formatter(pythonStyle=pythonStyle,
                                      sparksqlStyle=sparksqlAPI._create_style_from_dict(sparksqlStyle),
                                      queryNames=queryNames)
            else:
                formatter = Formatter(pythonStyle=pythonStyle,
                                      sparksqlStyle=sparksqlAPI._create_style_from_file(sparksqlStyle),
                                      queryNames=queryNames)
        elif type(sparksqlStyle) == dict:
            formatter = Formatter(pythonStyle=pythonStyle,
                                  sparksqlStyle=sparksqlAPI._create_style_from_dict(sparksqlStyle),
                                  queryNames=queryNames)
        else:
            raise Exception('Unsupported config type')
    return _format_script(script, formatter)


def _format_file(filePath, formatter, inPlace=False):
    '''
    The I/O helper function for format_file(). Read from given file, format it, and write to specified output.

    Parameters
    filePath: string
        Path to the file to format.
    formatter: pysqlformatter.src.formatter.Formatter() object
        Formatter.
    inPlace: bool
        If True, will format the file in place.
        Else, will write the formatted file to stdout.
    
    Return: None
    '''
    script = _read_from_file(filePath)
    formattedScript = _format_script(script, formatter)
    if inPlace:  # overwrite file
        logger.info('Writing to ' + filePath + '...')
        _write_to_file(formattedScript, filePath)
    else:  # write to stdout
        sys.stdout.write(formattedScript)


def _read_from_file(filePath):
    '''
    The input helper function for _format_file(). Read from given file and return its content.

    Parameters
    filePath: string
        Path to the file to format.
    
    Return: string
        The file content.
    '''
    # see https://docs.python.org/3.5/library/functions.html#open
    with open(file=filePath, mode='r', newline=None, encoding='utf-8') as f:
        text = f.read()
    return text


def _write_to_file(formattedQuery, filePath):
    '''
    The output helper function for _format_file(). Write formatted query to given file.

    Parameters
    formattedQuery: string
        The formatted query.
    filePath: string
        Path to the file to write to.
    '''
    # see https://docs.python.org/3.5/library/functions.html#open
    with open(file=filePath, mode='w', newline='\n', encoding='utf-8') as f:
        f.write(formattedQuery)


def _format_script(script, formatter):
    '''
    The wrapper function for format_script(). Format a given script using given formatter.

    Parameters
    string: string
        The script to format.
    formatter: sparksqlformatter.src.formatter.Formatter() object
        Formatter.
    
    Return: string
        The formatted script.
    '''
    return formatter.format(script)



# -*- coding: utf-8 -*-
# MIT License

# Copyright (c) 2020-present largecats

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# from __future__ import print_function  # for print() in Python 2
import os
import sys
import argparse
import logging
import codecs
import json
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from pysqlformatter.src import api
from pysqlformatter.src.formatter import Formatter

logger = logging.getLogger(__name__)
log_formatter = '[%(asctime)s] %(levelname)s [%(filename)s:%(lineno)s:%(funcName)s] %(message)s'
logging.basicConfig(stream=sys.stdout, level=logging.INFO, format=log_formatter)


def main(argv):
    args = get_arguments(argv)
    pythonStyle = args['python_style']
    sparksqlStyle = args['sparksql_style']
    filePaths = args['files']
    queryNames = args['query_names']
    if filePaths:
        if sparksqlStyle:
            if pythonStyle:
                for filePath in filePaths:
                    api.format_file(filePath=filePath,
                                    pythonStyle=pythonStyle,
                                    sparksqlStyle=sparksqlStyle,
                                    queryNames=queryNames,
                                    inPlace=args.get('in_place'))
            else:
                for filePath in filePaths:
                    api.format_file(filePath=filePath,
                                    sparksqlStyle=sparksqlStyle,
                                    queryNames=queryNames,
                                    inPlace=args.get('in_place'))
        else:
            if pythonStyle:
                for filePath in filePaths:
                    api.format_file(filePath=filePath,
                                    pythonStyle=pythonStyle,
                                    queryNames=queryNames,
                                    inPlace=args.get('in_place'))
            else:
                for filePath in filePaths:
                    api.format_file(filePath=filePath, queryNames=queryNames, inPlace=args.get('in_place'))


def get_arguments(argv):
    '''
    Return arguments passed via command-line.

    Parameters:
    argv: list
        sys.argv
    
    Returns: dict
        A dictionary containing arguments for the formatter.
    '''
    parser = argparse.ArgumentParser(description='Formatter for Pyspark code and SparkSQL queries.')

    parser.add_argument('-f', '--files', type=str, nargs='+', help='Paths to files to format.')

    parser.add_argument('-i', '--in-place', action='store_true', help='Format the files in place.')

    parser.add_argument('--python-style',
                        type=str,
                        default=None,
                        help='Style for Python formatting, interface to https://github.com/google/yapf.')

    parser.add_argument(
        '--sparksql-style',
        type=str,
        default=None,
        help="Style for SparkSQL formatting, interface to https://github.com/largecats/sparksql-formatter.")

    parser.add_argument(
        '--query-names',
        type=str,
        default=['query'],
        nargs='+',
        help="String variables with names containing these strings will be formatted as SQL queries. Default to 'query'."
    )

    args = vars(parser.parse_args(argv[1:]))

    return args


def run_main():
    main(sys.argv)


if __name__ == '__main__':
    run_main()



from __future__ import print_function  # for print() in Python 2
import re
from yapf.yapflib import yapf_api
from sparksqlformatter import Style
from sparksqlformatter import api as sparksqlformatter_api
from pysqlformatter.src.tokenizer import Tokenizer


class Formatter:
    '''
    Format a script with Python code and SparkSQL queries.
    '''
    def __init__(self, pythonStyle='pep8', sparksqlStyle=Style(), queryNames=['query']):
        '''
        Parameters
        pythonStyle: string
            A style name or path to a style config file; interface to https://github.com/google/yapf.
        sparksqlStyle: string, dict, or sparksqlformatter.src.style.Style() object
            Configurations for the query language; interface to https://github.com/largecats/sparksql-formatter.
        '''
        self.pythonStyle = pythonStyle
        self.sparksqlStyle = sparksqlStyle
        self.pointer = 0  # next position to read
        self.tokenizer = Tokenizer(queryNames=queryNames)

    def format(self, script):
        pythonReformatted = yapf_api.FormatCode(script, style_config=self.pythonStyle)[0]
        tokens = self.tokenizer.tokenize(pythonReformatted)  # get all strings passed to spark.sql() in the .py script
        return self.get_formatted_script_from_tokens(pythonReformatted, tokens)

    def get_formatted_script_from_tokens(self, script, tokens):
        '''
        Format the given script.

        Parameters
        scirpt: string
            The script to format.
        
        Return: string
            The formatted script.
        '''
        formattedScript = ''
        for token in tokens:
            formattedScript += script[self.pointer:token.start]
            formattedQuery = sparksqlformatter_api.format_query(
                token.value, self.sparksqlStyle)  # will get rid of starting/trailling blank spaces

            formattedQuery = Formatter.indent_query(formattedQuery, token.indent)
            if not script[(token.start - 3):token.start] in [
                    "'''", '"""'
            ]:  # handle queries quoted by '' or "" that are formatted to multiline
                if '\n' in formattedQuery:
                    formattedScript = formattedScript[:-1] + "'''\n"  # remove starting ' or "
                    formattedScript += formattedQuery
                    formattedScript += '\n' + token.indent + "'''"
                    self.pointer = token.end + 1  # skip ending ' or "
                else:
                    formattedScript += formattedQuery
                    self.pointer = token.end
            else:
                formattedScript += '\n' + formattedQuery + '\n' + token.indent  # properly format between triple quotes
                self.pointer = token.end
        formattedScript += script[self.pointer:]
        self.reset()
        return yapf_api.FormatCode(formattedScript, style_config=self.pythonStyle)[0]

    @staticmethod
    def indent_query(query, indent):
        '''
        Indent given query when formatting.

        Parameters
        query: string
            The query to indent.
        indent: string
            The indentation.
        
        Return: string
            The indented query.
        '''
        lines = query.split('\n')
        lines = [indent + line for line in lines]
        indentedQuery = '\n'.join(lines)
        return indentedQuery

    def reset(self):
        '''
        Reset the formatter after finishing formatting a script.
        '''
        self.pointer = 0



import re

from pysqlformatter.src.token import TokenType, Token


class Tokenizer:
    def __init__(self, queryNames):
        '''
        Class for tokenizing the script to be formatted.

        Parameters
        queryNames: list
            Strings used to identify variables that contain the SparkSQL queries. 
            All string variables whose name contain these strings will be formatted.
        '''
        self.queryNames = queryNames

    def tokenize(self, script):
        '''
        Extract all queries from script.

        Parameters
        script: string
            The script to format.
        
        Return: list
            Query tokens found in the script.
        '''
        queryMatchTokens = self.get_queries(script=script)
        return queryMatchTokens

    def get_queries(self, script):
        queryVariableTokens = self.get_query_variable(script=script)
        queryArgumentTokens = self.get_query_argument(script=script)
        queryTokens = queryArgumentTokens + queryVariableTokens
        queryTokens.sort(key=lambda token: token.start)
        return queryTokens

    def get_query_variable(self, script):
        '''
        Find queries stored in variables with designated set of variable names in self.queryNames.

        Paramters
        script: string
            The script to format.
        
        Return: list
            Tokens of matched queries.
        '''
        queryVariableNames = '|'.join(self.queryNames)
        queryRegexSingleQuotes = '{queryVariableNames}\s*=\s*\'(.*?)\''.format(queryVariableNames=queryVariableNames)
        queryRegexDoubleQuotes = '{queryVariableNames}\s*=\s*"(.*?)"'.format(queryVariableNames=queryVariableNames)
        queryRegexTipleSingleQuotes = '{queryVariableNames}\s*=\s*\'\'\'(.*?)\'\'\''.format(
            queryVariableNames=queryVariableNames)
        queryRegexTripleDoubleQuotes = '{queryVariableNames}\s*=\s*"""(.*?)"""'.format(
            queryVariableNames=queryVariableNames)
        queryRegex = '|'.join(
            [queryRegexTipleSingleQuotes, queryRegexTripleDoubleQuotes, queryRegexSingleQuotes, queryRegexDoubleQuotes])
        queryMatchObjs = re.finditer(pattern=queryRegex, string=script, flags=re.IGNORECASE | re.DOTALL)
        queryMatchList = [
            Tokenizer.create_token_from_match(matchObj=m, script=script, type=TokenType.QUERY_VARIABLE)
            for m in queryMatchObjs
        ]
        return queryMatchList

    def get_query_argument(self, script):
        '''
        Find queries in spark.sql() in the script.

        Paramters
        script: string
            The script to format.
        
        Return: list
            Tokens of matched queries.
        '''
        sparkSqlRegex = 'spark\.sql\((.*?)\)'
        sparkSqlMatchObjs = re.finditer(sparkSqlRegex, script, flags=re.DOTALL)
        queryMatchTokens = []
        for matchObj in sparkSqlMatchObjs:
            matchGroupIndex = 1  # we want the start of the matched group, so need to skip m.start(0), the entire matched string
            for m in matchObj.groups():
                if m:
                    matchGroup = m  # first match group that is not None
                    break
                else:
                    matchGroupIndex += 1
            if Tokenizer.is_query(matchGroup):  # e.g., spark.sql('select * from t0')
                if matchGroup.startswith("'''") or matchGroup.startswith('"""'):  # enclosed by triple quotes
                    matchTokenWithoutQuotes = Token(
                        type=TokenType.QUERY_ARGUMENT,
                        value=matchGroup.strip('"').strip("'"),
                        start=matchObj.start(matchGroupIndex) + 3,  # skip opening triple quotes
                        end=matchObj.end(matchGroupIndex) - 3,  # skip ending triple quotes
                        script=script)
                else:  # enclosed by single quotes
                    matchTokenWithoutQuotes = Token(
                        type=TokenType.QUERY_ARGUMENT,
                        value=matchGroup.strip('"').strip("'"),
                        start=matchObj.start(matchGroupIndex) + 1,  # skip opening quotes
                        end=matchObj.end(matchGroupIndex) - 1,  # skip ending quotes
                        script=script)
                queryMatchTokens.append(matchTokenWithoutQuotes)
            else:  # e.g., spark.sql(query)
                continue  # ignore, because it is already handled by self.get_query_argument()
        return queryMatchTokens

    @staticmethod
    def is_query(text):
        '''
        Determine whether the argument in spark.sql() is a query.

        Parameters
        text: string
            The content in spark.sql()
        
        Return: bool
            True if the text is a query. E.g., spark.sql('select * from t0').
            False if the text is a variable that stores the query. E.g., spark.sql(query)
        '''
        return text.startswith("'") or text.startswith('"')

    @staticmethod
    def remove_quotes(query):
        '''
        Remove starting and ending quotes from query.

        Parameters
        query: string
            The query to format.
        
        Return: string
            Query without starting and ending quotation marks.
        '''
        return query.strip('"').strip("'")

    @staticmethod
    def create_token_from_match(matchObj, script, type):
        '''
        Create token from re.match object.

        Parameters
        matchObj: re.match object
        script: string
            The script to format.
        type: string
            TokenType.
        
        Return: Token() object
        '''
        matchGroupIndex = 1
        for m in matchObj.groups():
            if m:
                return Token(
                    type=type,
                    value=m,
                    start=matchObj.start(
                        matchGroupIndex
                    ),  # start(0) is the start of the whole match, see https://docs.python.org/3/library/re.html re.Match.start([group])
                    end=matchObj.end(matchGroupIndex),  # end(0) is the end of the whole match
                    script=script)
            else:
                matchGroupIndex += 1



class TokenType:
    QUERY_VARIABLE = 'QUERY_VARIABLE'
    QUERY_ARGUMENT = 'QUERY_ARGUMENT'


class Token:
    def __init__(self, type, value, start, end, script):
        '''
        Parameters:
        type: string
            TokenType.
        value: string
        start: int
            Position of the token's first character.
        end: int
            Position of the token's last character.
        script: string
            The script containing the token.
        '''
        __slots__ = 'value', 'start', 'end', 'type'  # saves space since there would be many instances of Token
        self.value = value
        self.start = start
        self.end = end
        self.type = type
        self.indent = self.get_token_indent(start, script)

    def get_token_indent(self, start, script):
        '''
        Determine the token's indentation when formatting the script.

        Parameters
        start: int
            Start of the token.
        script: string
            The script containing the token.
        
        Return: string
            The token's indentation.
        '''
        queryStart = Token.get_query_start(start, script)
        currLineIndent = Token.get_line_indent(queryStart, script)
        prevLineEnd = Token.get_prev_line_end(queryStart, script)
        prevLineIndent = Token.get_line_indent(prevLineEnd, script)
        if queryStart == start:  # if there are no starting \n, align with current line
            indent = currLineIndent
        else:  # align with previous line
            indent = prevLineIndent
        # indent = currLineIndent if len(currLineIndent) > len(prevLineIndent) else prevLineIndent
        return indent

    @staticmethod
    def get_line_indent(pos, script):
        '''
        Get indentation of the line of given position in script.

        Parameters
        pos: int
            Position to get indentation for.
        script: string
            Script to format.
        
        Return: string
            Indentation of the line containing given position.
        '''
        startOfLine = pos
        while startOfLine > 0 and (script[startOfLine] != '\n'):  # find start of line
            startOfLine -= 1
        startOfLine += 1
        indent = 0
        # find position of the first non-space character in the line
        while (script[startOfLine + indent] != '\n') and script[startOfLine + indent].isspace():
            indent += 1
        indentString = script[startOfLine:startOfLine + indent]
        return indentString

    @staticmethod
    def get_query_start(pos, script):
        '''
        Get position of the start of the first line of the actual query in a query match.

        Parameters
        pos: int
            Position to get indentation for.
        script: string
            Script to format.
        
        Return: int
            Position of the start of the first line of the query.
        '''
        startOfQuery = pos
        while script[startOfQuery] == '\n':  # skip \n after the opening '''/"""
            startOfQuery += 1
        return startOfQuery

    @staticmethod
    def get_prev_line_end(pos, script):
        '''
        Get position of the end of the previous non-empty line.

        Parameters
        pos: int
            Position to get indentation for.
        script: string
            Script to format.

        Return: int
            Position of the last character in the previous line.
        '''
        endOfPrevLine = pos
        while endOfPrevLine > 0 and script[endOfPrevLine] != '\n':
            endOfPrevLine -= 1
        while script[endOfPrevLine] == '\n':
            endOfPrevLine -= 1
        return endOfPrevLine


import sys
import os
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
import logging

from pysqlformatter.src import api

logger = logging.getLogger(__name__)
log_formatter = '[%(asctime)s] %(levelname)s [%(filename)s:%(lineno)s:%(funcName)s] %(message)s'
logging.basicConfig(stream=sys.stdout, level=logging.INFO, format=log_formatter)


class Test:
    def __init__(self):
        pass

    def test_script_with_simple_query_as_variable(self):
        msg = 'Testing script with simple query as variable passed to spark.sql()'
        testScript = """
query = 'select * from t0'
df = spark.sql(query)
        """
        key = """
query = '''
SELECT
    *
FROM
    t0
'''
df = spark.sql(query)
        """.strip() + '\n'  # pep8
        self.run(msg, testScript, key)

    def test_script_with_simple_query_as_argument(self):
        msg = 'Testing script with simple query in spark.sql()'
        testScript = """
df = spark.sql('select * from t0')
        """
        key = """
df = spark.sql('''
SELECT
    *
FROM
    t0
''')
        """.strip() + '\n'  # pep8
        self.run(msg, testScript, key)

    def test_script_with_complex_query_as_variable1(self):
        msg = 'Testing script with complex query in spark.sql()'
        testScript = """
import re

def get_base(date):
    query = '''
    select * from
    t0
    left join t1 on t0.id = t1.id
    where t1.date = '{date}'
    '''
    query = query.format(date=date)
    df = spark.sql(query)
    df.cache()
    return df

def add_columns():
    df = spark.sql('select * from base left join t2 on base.id = t2.id')
    return df

def final():
    columns = ['id', 'date', 'c1', 'c2']
    df = df.select(columns)
    return df
        """
        key = """
import re


def get_base(date):
    query = '''
    SELECT
        *
    FROM
        t0
    LEFT JOIN
        t1
        ON t0.id = t1.id
    WHERE
        t1.date = '{date}'
    '''
    query = query.format(date=date)
    df = spark.sql(query)
    df.cache()
    return df


def add_columns():
    df = spark.sql('''
    SELECT
        *
    FROM
        base
    LEFT JOIN
        t2
        ON base.id = t2.id
    ''')
    return df


def final():
    columns = ['id', 'date', 'c1', 'c2']
    df = df.select(columns)
    return df
        """.strip() + '\n'  # pep8
        self.run(msg, testScript, key)

    def test_script_with_complex_query_as_variable_currLineIndent_less_than_prevLineIndent(self):
        msg = 'Testing script with complex query in spark.sql(), with currLineIndent < prevLineIndent'
        testScript = """
import re

def get_base(date):
    query = '''
select * from
t0
left join t1 on t0.id = t1.id
where t1.date = '{date}'
    '''
    query = query.format(date=date)
    df = spark.sql(query)
    df.cache()
    return df

def add_columns():
    df = spark.sql('select * from base left join t2 on base.id = t2.id')
    return df

def final():
    columns = ['id', 'date', 'c1', 'c2']
    df = df.select(columns)
    return df
        """
        key = """
import re


def get_base(date):
    query = '''
    SELECT
        *
    FROM
        t0
    LEFT JOIN
        t1
        ON t0.id = t1.id
    WHERE
        t1.date = '{date}'
    '''
    query = query.format(date=date)
    df = spark.sql(query)
    df.cache()
    return df


def add_columns():
    df = spark.sql('''
    SELECT
        *
    FROM
        base
    LEFT JOIN
        t2
        ON base.id = t2.id
    ''')
    return df


def final():
    columns = ['id', 'date', 'c1', 'c2']
    df = df.select(columns)
    return df
        """.strip() + '\n'  # pep8
        self.run(msg, testScript, key)

    def test_script_with_complex_query_as_variable_currLineIndent_greater_than_prevLineIndent(self):
        msg = 'Testing script with complex query in spark.sql(), with currLineIndent > prevLineIndent'
        testScript = """
import re

def get_base(date):
    query = '''
        select * from
        t0
        left join t1 on t0.id = t1.id
        where t1.date = '{date}'
    '''
    query = query.format(date=date)
    df = spark.sql(query)
    df.cache()
    return df

def add_columns():
    df = spark.sql('select * from base left join t2 on base.id = t2.id')
    return df

def final():
    columns = ['id', 'date', 'c1', 'c2']
    df = df.select(columns)
    return df
        """
        key = """
import re


def get_base(date):
    query = '''
    SELECT
        *
    FROM
        t0
    LEFT JOIN
        t1
        ON t0.id = t1.id
    WHERE
        t1.date = '{date}'
    '''
    query = query.format(date=date)
    df = spark.sql(query)
    df.cache()
    return df


def add_columns():
    df = spark.sql('''
    SELECT
        *
    FROM
        base
    LEFT JOIN
        t2
        ON base.id = t2.id
    ''')
    return df


def final():
    columns = ['id', 'date', 'c1', 'c2']
    df = df.select(columns)
    return df
        """.strip() + '\n'  # pep8
        self.run(msg, testScript, key)

    def test_script_with_query_starting_with_comment(self):
        msg = 'Testing script with query that starts with line comment'
        testScript = """
def foo():
    query = '''
    -- this is a line comment
    select * from t0
    '''
    return spark.sql(query)
        """
        key = """
def foo():
    query = '''
    -- this is a line comment
    SELECT
        *
    FROM
        t0
    '''
    return spark.sql(query)
        """.strip() + '\n'  # pep8
        self.run(msg, testScript, key)

    def run(self, msg, testScript, key):
        logger.info(msg)
        logger.info('testScript =')
        logger.info(testScript)
        logger.info(repr(testScript))
        formattedScript = api.format_script(testScript)
        logger.info('formattedScript =')
        logger.info(formattedScript)
        logger.info(repr(formattedScript))
        logger.info('key =')
        logger.info(key)
        logger.info(repr(key))
        assert formattedScript == key
        return True

    def run_all(self):
        tests = list(filter(lambda m: m.startswith('test_'), dir(self)))
        for test in tests:
            getattr(self, test)()


if __name__ == "__main__":
    Test().run_all()



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(name='pysqlformatter',
                 version='0.0.4',
                 author='largecats',
                 author_email='linfanxiaolinda@outlook.com',
                 description='A formatter for Python code and SparkSQL queries.',
                 long_description=long_description,
                 long_description_content_type='text/markdown',
                 url='https://github.com/largecats/pyspark-sql-formatter',
                 packages=setuptools.find_packages(),
                 install_requires=['yapf', 'sparksqlformatter', 'configparser'],
                 classifiers=[
                     'Programming Language :: Python',
                     'Programming Language :: Python :: 2',
                     'Programming Language :: Python :: 2.7',
                     'Programming Language :: Python :: 3',
                     'Programming Language :: Python :: 3.6',
                     'License :: OSI Approved :: MIT License',
                     'Operating System :: OS Independent',
                 ],
                 entry_points={
                     'console_scripts': ['pysqlformatter=pysqlformatter:run_main'],
                 })
